@@ -58,6 +58,16 @@ dimCheck <- function(a, b) {
     da
 }
 
+mmultCheck <- function(a, b) {
+    ca <- dim(a)[2]
+    rb <- dim(b)[1]
+    if(ca != rb)
+	stop(gettextf("non-conformable matrix dimensions in %s",
+		      deparse(sys.call(sys.parent()))),
+	     call. = FALSE)
+    ca
+}
+
 dimNamesCheck <- function(a, b) {
     ## assume dimCheck() has happened before
     nullDN <- list(NULL,NULL)
@@ -133,6 +143,17 @@ emptyColnames <- function(x, msg.if.not.empty = FALSE)
     x
 }
 
+idiag <- function(n, p=n)
+{
+    ## Purpose: diag() returning  *integer*
+    ## --------------------------------------------------------
+    ## Author: Martin Maechler, Date:  8 Dec 2007, 23:13
+    r <- matrix(0L, n,p)
+    if ((m <- min(n, p)) > 0)
+	r[1 + 0:(m - 1) * (n + 1)] <- 1L
+    r
+}
+
 ### TODO:  write in C and port to base (or 'utils') R
 indTri <- function(n, upper = TRUE) {
     ## == which(upper.tri(diag(n)) or
@@ -290,6 +311,7 @@ non0ind <- function(x, classDef.x = getClassDef(class(x)))
 encodeInd  <- function(ij,  nr) ij[,1] + ij[,2] * nr
 encodeInd2 <- function(i,j, nr) i      +  j     * nr
 
+## 'code' is 0-based:
 decodeInd <- function(code, nr) cbind(code %% nr, code %/% nr)
 
 complementInd <- function(ij, dim)
@@ -637,8 +659,8 @@ as_dense <- function(x, cld = if(isS4(x)) getClassDef(class(x))) {
 
 
 ### Goal: Eventually get rid of these --- want to foster coercions
-### ----  *to* virtual classes whenever possible, i.e.
-##  as(*, "CsparseMatrix"),  etc
+### ----  *to* virtual classes whenever possible, e.g. as(*, "CsparseMatrix")
+## 2007-12: better goal: use them only for "matrix" [maybe speed them up later]
 
 ## Here, getting the class definition and passing it, should be faster
 as_Csparse <- function(x, cld = if(isS4(x)) getClassDef(class(x))) {
@@ -653,6 +675,13 @@ as_Csparse2 <- function(x, cld = if(isS4(x)) getClassDef(class(x))) {
     if(sh == "t") .Call(Csparse_diagU2N, x) else x
 }
 
+## *do* use this where applicable
+as_Csp2 <- function(x) {
+    ## Csparse + U2N when needed
+    x <- as(x, "CsparseMatrix")
+    if(is(x, "triangularMatrix")) .Call(Csparse_diagU2N, x) else x
+}
+
 
 ## 'cl'   : class() *or* class definition of from
 as_gCsimpl2 <- function(from, cl = class(from))
