@@ -70,7 +70,7 @@ is.all.equal4 <- function(x,y,z,u, tol = .Machine$double.eps^0.5)
 
 
 Qidentical <- function(x,y) {
-    ## quasi-identical:
+    ## quasi-identical - for 'Matrix' matrices
     if(class(x) != class(y)) return(FALSE)
     slts <- slotNames(x)
     if("factors" %in% slts) { ## allow one empty and one non-empty 'factors'
@@ -85,6 +85,17 @@ Qidentical <- function(x,y) {
     TRUE
 }
 
+## A version of all.equal() for the slots
+all.slot.equal <- function(x,y, ...) {
+    slts <- slotNames(x)
+    for(sl in slts) {
+        aeq <- all.equal(slot(x,sl), slot(y,sl), ...)
+        if(!identical(TRUE, aeq))
+            return(paste("slot '",sl,"': ", aeq, sep=''))
+    }
+    TRUE
+}
+
 ## The relative error typically returned by all.equal:
 relErr <- function(target, current)
     mean(abs(target - current)) / mean(abs(target))
